#include "unistd.h"

#include <string.h>
#include <stdio.h>

char *optarg;
int opterr = 1, optind = 1, optopt;

int getopt(int argc, char *const argv[], const char *optstring)
{
    /* If optlast == argv[optind], then we are part way through parsing
     * argv[optind] and the next character to analyse is *optchar.
    */
    static char *optlast = 0, *optchar = 0;
    const char *findchr;

    optarg = 0;
    optopt = 0;
    if (argv[optind] == 0 || argv[optind][0] != '-' || argv[optind][1] == 0)
    {
        /* End of options. */
        return -1;
    }
    if (strcmp(argv[optind], "--") == 0)
    {
        /* Option / operand separator. */
        optind++;
        return -1;
    }

    /* Get the option character. */
    if (optlast != argv[optind])
    {
        optlast = argv[optind];
        optchar = optlast + 1;
    }
    optopt = *optchar;
    if (!*++optchar)
        optchar = argv[++optind];

    if (findchr = strchr(optstring, optopt))
    {
        if (findchr[1] == ':')
        {
            if (++optind > argc)
            {
                /* Missing argument. */
                if (optstring[0] == ':')
                {
                    return ':';
                }
                else
                {
                    if (opterr)
                        fprintf(stderr, "%s - option -%c requires an operand", argv[0], optopt);
                    return '?';
                }
            }
            optarg = optchar;
        }
        return optopt;
    }
    else
    {
        /* Unknown option. */
        if (opterr && optstring[0] != ':')
            fprintf(stderr, "%s - invalid option -%c\n", argv[0], optopt);
        return '?';
    }
}
