// flip.cpp : Implementation of main().

#include "flip.h"
#include "flip-curses.h"
#include "flip-std.h"

#include <unistd.h>

#include <iostream>
#include <cstdlib>

int main(int argc, char* argv[])
{
    char mode = ' ';
    flip_options options = {NULL, NULL, 50, false};
    int c, result = -1;
    
#ifdef FLIP_CURSES_SUPPORT
# define OPT_C "c"
#else
# define OPT_C
#endif
    while ((c = getopt(argc, argv, OPT_C "hi:o:s:u")) != -1)
    {
        switch (c)
        {
#ifdef FLIP_CURSES_SUPPORT
        case 'c':
            mode = 'c';
            break;
#endif
        case 'h':
            if (result < 0)
                result = 0;
            break;
        case 'i':
            options.infile = optarg;
            break;
        case 'o':
            options.outfile = optarg;
            break;
        case 's':
            options.speed = std::atol(optarg);
            break;
        case 'u':
            options.unspectest = true;
            break;
        case '?':
            result = 1;
            break;
        }
    }
    if (result < 0 && optind > argc - 1)
    {
        std::cerr << argv[0] << ": Not enough arguments" << std::endl;
        result = 1;
    }
    else if (result < 0 && optind < argc - 1)
    {
        std::cerr << argv[0] << ": Too many arguments" << std::endl;
        result = 1;
    }
    if (result >= 0)
    {
        std::cout << argv[0] << " - interpreter for the Flip programming language.\n"
            "Usage:\n"
            "    " << argv[0] << " [-c] [-i file] [-o file] [-s delay] [-u] source-file\n"
            "    " << argv[0] << " -h\n"
            "Options:\n"
#ifdef FLIP_CURSES_SUPPORT
            "  -c    Show the program excecution in the console.\n"
#else
            " (-c)   This build does not support the -c option.\n"
#endif
            "  -h    Show this help and exit.\n"
            "  -i file\n"
            "        Read input from the specified file.\n"
            "  -o file\n"
            "        Write output to the specified file.\n"
            "  -s delay\n"
            "        Set the excecution speed (only affects -c).\n"
            "  -u    Test for unspecified behaviour (anything which depends on the order\n"
            "        balls are processed in within each cycle)." << std::endl;
        return result;
    }
    char* sourcefile = argv[optind++];
    
    switch (mode)
    {
    case ' ':
        result = flip_std(sourcefile, options);
        break;
#ifdef FLIP_CURSES_SUPPORT
    case 'c':
        result = flip_curses(sourcefile, options);
        break;
#endif
    }
    
    return result;
}
